c      este programa e um pouquinho mais inteligente que o outro,
c      usa matrizes pra testar todos E(B-V) ao mesmo tempo
c           Population Synthesis Program
c           Version 7
c           Henrique R. Schmitt
c           SUN-Version November, 1996
c           This version includes lines, continua and reddening
c     It uses a reduced base, including only 7 clusters of different
c     ages and metallicities, as defined by Schmidt et al. (1991)
c
c
c     AVAILABLE COMPONENTS ARE:
c     AGE(Myr): rhii 10 50 100 400 2000 E2(bulge) 
c                 1   2  3   4  5    6      7
c
c     In this version is possible to calculate the synthesis for
c     1 dIfferent spectra at the same time, using 51 E(B-V) steps
c     of 0.01, from 0.0 to 0.5.
c
c     You have to provide the measured equivalent widths of 9 lines:
c     (1)-SiII1309 (2)-SiIV1400 (3)-CIV1549  (4)-NV1719 (5)-FeI2522
c     (6)-MnII2580 (7)-FeII2610 (8)-MgII2800 (9)-MgI2852
c
c     You also have to give 9 continuum ratios relative to 2646A:
c     1282 1348 1490 1768 2079 2258 2466 2959 3122
c     
c     The output file gives the number of combinations, the number of
c     solutions, contribution from each age and metallicity component
c     and their standard deviation, the mean deviation from the 
c     observed equivalent width or continuum ratio relative to the 
c     predicted one.
c     It also gives you the solution averaged relative to the number
c     of solution in each E(B-V), average E(B-V).
c
c      implicit none

      integer*4 i,j,t,w,k,sum,lim,ncont(51,1),contf(1)
      integer*4 Step,ind,sqnb(7,51,1),nb(7,51,1)
      integer*4 ik,ij,SC(7),nbf(7,1),sqnbf(7,1),redL
      Integer*8 nComb

c       nComb,    number of combinations
c       ncont,    number of solutions
c       na,       number of clusters
c       nb(44),
c       sqnb(44),
c       nr,       number of spectral lines
c       nc,       number of continuum points
c       SC(44),   percentage contribution of cluster i
c       Min(44) ,  lower limit for contribution i
c       Max(44),   upper limit for contribution i
c       Step,       step for cobtribution variation
c       ind,

      real*4 CA(7,8),WA(7,8),WG(8,1),ebf(51),DR(8),tldv(7,1),
     *CSG(8,51,1),CSA(7,8),DC(8),tcdv(7,1),sir(8),Cont(8,51,1),
     *moy(7,51,1),moysq(7,51,1),moycont(8,51,1),sqrvar(7,1),
     *moymarg(8,51,1),cor(8),CSGI(8,1),DL(8,1),tmoy(7,1),
     *marg(8,51,1),errcon(8,51,1),errcoi(8,1),tmoymg(7,1),
     *tmoysq(7,1),tmoyct(7,1),tsqrv(7,1),ebvtf(1),ebvtt(1),
     *sqeb(1),sqveb(1),Cvar,cor_LMC(8),cor_SMC(8),cor_GAL(8)

c       TabCA.dat cluster i contin at spctr lin j
c       CA(44,9),
c       TabWA.dat cluster i equiv. width of line j
c       WA(44,9),
c       WG(9),spectral line equiv. width for object
c       CSG(7),
c       CSA(44,7),
c       TabCSA.dat cluster continua
c       DC(7),   computed continuum deviation
c       errcon, allowed continuum deviation
c       DR(9),   computed eq. width deviation
c       DL(9),   allowed eq. width deviation
c       sir(9),  sigma for spectral lines
c       marg(9),
c       Cont(7),
c       moy(44), mean of contributions for each cluster
c       moysq(44),
c       moymarg(9),
c       moycont(7)

        common errcoi
        common/com/ nSyn
        common/com1/SC
        common/com2/Step
        common/com3/WG,CSG,WA,CA,CSA
        common/com4/DL,errcon
        common/com5/nComb
c######################################################################
c       read(*,*) nSyn

       nSyn = 1

       Do ki=1,nSyn
          read(*,*) (wg(i,ki),i=1,8)
          read(*,*) (dl(i,ki),i=1,8)
          read(*,*) (CSGI(j,ki),j=1,8)
          read(*,*) (errcoi(j,ki),j=1,8)
       EndDo

       open(1,file='TabWA.dat')
       read(1,*)((WA(ij,ik),ik=1,8,1),ij=1,7,1)
       open(2,file='TabCA.dat')
       read(2,*)((CA(ij,ik),ik=1,8,1),ij=1,7,1)
       open(4,file='TabCSA.dat')
       read(4,*)((CSA(ij,ik),ik=1,8,1),ij=1,7,1)

       Step=5


c************************************************
c       LMC EXTINCTION LAW VALUES
************************************************
c
c       redL = 1
c       cor_LMC(1)=2.4868
c       cor_LMC(2)=2.1456
c       cor_LMC(3)=1.6008
c       cor_LMC(4)=1.0368
c       cor_LMC(5)=1.0552
c       cor_LMC(6)=0.8564
c       cor_LMC(7)=0.2824
c       cor_LMC(8)=-0.2996
c       cor_LMC(9)=-0.4148
c
c***********************************************
c       SMC EXTINCTION LAW VALUES
c***********************************************
c
c       redL = 2
c       cor_SMC(1)=3.896
c       cor_SMC(2)=3.388
c       cor_SMC(3)=2.460
c       cor_SMC(4)=1.563
c       cor_SMC(5)=0.8615
c       cor_SMC(6)=0.5462
c       cor_SMC(7)=0.2357
c       cor_SMC(8)=-0.3334
c       cor_SMC(9)=-0.482223
c
c

c***********************************************
c       GALACTIC EXTINCTION LAW VALUES
c***********************************************

       redL = 3
       cor_GAL(1)=0.3382
       cor_GAL(2)=0.2366
       cor_GAL(3)=0.1777
       cor_GAL(4)=0.1724
       cor_GAL(5)=0.1028
       cor_GAL(6)=0.0212
       cor_GAL(7)=0.0140
       cor_GAL(8)=0.0001

c       redL = 3

       Do i=1,8
          If (redL .eq. 1) Then
             cor(i) = cor_LMC(i)
          EndIf

          If (redL .eq. 2) Then
             cor(i) = cor_SMC(i)
          EndIf

          If (redL .eq. 3) Then
             cor(i) = cor_GAL(i)
          EndIf
       EndDo
       
c###################################################################

        Do ki=1,nSyn
           contf(ki)=0
           ebvtf(ki)=0.
           ebvtt(ki)=0.
           sqeb(ki)=0.
           sqveb(ki)=0.

           Do j=1,51
              ebf(j)=(float(j)-1.)*0.01
              ncont(j,ki)=0
             
              Do k=1,8
                 marg(k,j,ki)=0.
                 Cont(k,j,ki)=0.
                 CSG(k,j,ki)=CSGI(k,ki)*10.**(cor(k)*ebf(j))
                 errcon(k,j,ki)=errcoi(k,ki)*10.**(cor(k)*ebf(j))
              EndDo
           EndDo  

           Do i=1,7
              SC(i)=0
              sqrvar(i,ki)=0.
              nbf(i,ki)=0
              sqnbf(i,ki)=0.
              tldv(i,ki)=0.
              tcdv(i,ki)=0.
              tmoy(i,ki)=0.
              tmoymg(i,ki)=0.
              tmoysq(i,ki)=0.
              tsqrv(i,ki)=0.
              tmoyct(i,ki)=0.
       
              Do l=1,51
                 nb(i,l,ki)=0
                 sqnb(i,l,ki)=0
                 moysq(i,l,ki)=0.
              EndDo
           EndDo
        EndDo

        nComb=0
        SUM=0
        lim=100
        w=0

c#####################################################################

        Do i=1,7
           ind=i
           t=i

1          SC(t)=SC(t)+Step
           SUM=0

           Do j=1,7
              SUM=SUM+SC(j)
           EndDo

           If ((SC(t).ne.100).and.(SUM.ne.Lim).and.(w.ne.1)) Go To 1

           If ((SC(t).eq.100).and.(SUM.ne.Lim)) Then
              If (t.eq.ind) Then
                 If (w.ne.1) Then
                    Call Remise(t,SC)
                    w=1
                    Go To 10
                 Else
                    Go To 3
                 EndIf
              EndIf
              If (t.ne.ind) Then
                      Call Remise(t,SC)
                      t=t+1
                      w=1
                      Go To 1
              EndIf
           EndIf

           If ((SC(t).eq.100).and.(SUM.eq.Lim)) Then
              Call Traitement(SC,ncont,nb,marg,Cont,DR,DC,sir,sqnb)
              nComb=nComb+1
              Call Remise(t,SC)
              If (t.eq.7) Go To 2
              If (t.eq.ind) Then
                 w=1
                 Go To 10
              EndIf
              If (t.ne.ind) Then
                t=t+1
                w=1
                Go To 1
              EndIf
           EndIf

            If ((SC(t).ne.100).and.(SUM.eq.Lim)) Then
               Call Traitement(SC,ncont,nb,marg,Cont,DR,DC,sir,sqnb)
               nComb=nComb+1
               If (SC(t+1).eq.100) Then
                  If (t+1.eq.ind) Then
                      Call remise(ind,SC)
                      w=1
                      Go To 10
                  EndIf
                  Call remise(t+1,SC)
                  t=t+2
                  w=1
                  Go To 1
               EndIf
               Call remise(t,SC)
               w=1
               t=t+1
               Go To 1
           EndIf

3          If (w.eq.1) Then
              t=1
              w=0
                Go To 1
           EndIf
10      EndDo
2       continue

        Do ml=1,nSyn
           Do k=1,51

              If (ncont(k,ml).ne.0) Then
                 contf(ml)=contf(ml)+ncont(k,ml)

                 Do i=1,7
                    moy(i,k,ml)=Float(nb(i,k,ml))/Float(ncont(k,ml))
                    moysq(i,k,ml)=float(sqnb(i,k,ml))/float(ncont(k,ml))
                    sqrvar(i,ml)=moysq(i,k,ml)-(moy(i,k,ml)*moy(i,k,ml))
                    sqrvar(i,ml)=(sqrvar(i,ml))**0.5
                    nbf(i,ml)=nbf(i,ml)+nb(i,k,ml)
                    sqnbf(i,ml)=sqnbf(i,ml)+sqnb(i,k,ml)
                 EndDo

                 Do j=1,8
                    moymarg(j,k,ml)=marg(j,k,ml)/(FLOATJ(ncont(k,ml)))
                    moycont(j,k,ml)=Cont(j,k,ml)/(FLOATJ(ncont(k,ml)))
                    tldv(j,ml)=tldv(j,ml)+marg(j,k,ml)
                    tcdv(j,ml)=tcdv(j,ml)+cont(j,k,ml)
                 EndDo

                 ebvtt(ml)=ebvtt(ml)+ncont(k,ml)*ebf(k)
                 sqeb(ml)=sqeb(ml)+ncont(k,ml)*ebf(k)*ebf(k)
              Else
                 Write(*,*)'No solution found'
              EndIf
        
              Write(*,*)'                          '
              Write(*,*)'                          '
              Write(*,752) ebf(k)
              Write(*,700) nComb
              Write(*,750) ncont(k,ml)

              If (ncont(k,ml).ne.0) Then
                 Write(*,*)
                 Write(*,*)'           *********************'
                 Write(*,*)'          **** MEAN SOLUTION ****'
                 Write(*,*)'           *********************'
                 Write(*,*)
                 Write(*,850) (moy(i,k,ml),i=1,7)
                 Write(*,851) (sqrvar(i,ml),i=1,7)
                 Write(*,*)

                 Cvar = 0.0
                 Do jj=1,8
                    Cvar = Cvar + moycont(jj,k,ml)*moycont(jj,k,ml) 
                 EndDo
                 Cvar = (Cvar)**0.5

                 Write(*,950) (wg(i,1),i=1,8)
                 Write(*,949) (moymarg(ii,k,ml),ii=1,8)
                 Write(*,*)
                 Write(*,951) (csg(i,k,ml),i=1,8)
                 Write(*,952) (moycont(jj,k,ml),jj=1,8)
                 Write(*,953) Cvar
              EndIf

              Write(*,*)
              Write(*,955)
           EndDo

           Write(*,*)
           Write(*,*)'-----------------------------------------------'
           Write(*,*)'                 AVERAGE VALUES'
           Write(*,*)'-----------------------------------------------'
           Write(*,*)

                   
           Do i=1,7
              tmoy(i,ml)=float(nbf(i,ml))/float(contf(ml))
              tmoysq(i,ml)=float(sqnbf(i,ml))/float(contf(ml))
              tsqrv(i,ml)=tmoysq(i,ml)-(tmoy(i,ml)*tmoy(i,ml ))
c              tsqrv(i,ml)=tsqrv(i,ml)**0.5
              tsqrv(i,ml)=(((tsqrv(i,ml))**2)**0.5)**0.5

           EndDo
           
           Write(*,*)"Esta Ok!!"          
        
           Do j=1,8
              tmoymg(j,ml)=tldv(j,ml)/(floatj(contf(ml)))
              tmoyct(j,ml)=tcdv(j,ml)/(floatj(contf(ml)))
           EndDo
           
                              
           ebvtf(ml)=ebvtt(ml)/float(contf(ml))
           sqveb(ml)=(sqeb(ml)-ebvtf(ml)*ebvtf(ml))/float(contf(ml))
           sqveb(ml)=sqveb(ml)**0.5
           Write(*,*)'E(B-V) = ',ebvtf(ml),'+-',sqveb(ml)
           Write(*,750) contf(ml)
           Write(*,*)
           Write(*,*) 'FINAL MEAN SOLUTION'
           Write(*,*)
           Write(*,850) (tmoy(i,ml),i=1,7)
           Write(*,851) (tsqrv(i,ml),i=1,7)
           Write(*,*)
           Write(*,950) (wg(i,1),i=1,8)
           Write(*,949) (tmoymg(ii,ml),ii=1,8)
           Write(*,*)
           Write(*,954) (tmoyct(jj,ml),jj=1,8)
        EndDo
        
          Write(*,*)"Esta Ok!!"

300     STOP

700     Format(/' Number of combinations='i10)
750     Format(' Number of solutions='i6)
752     Format('E(B-V) = '(F4.2))
850     Format('<Element contrib. (%)>: '10(f5.2,2X))
851     Format('      (+/-)           : '10(f5.2,2X))
949     Format('  <Residuals>   : '(1x,10f6.2))
950     Format('<EW(lines) in A>: '(1x,10f6.2))
951     Format('<Continua> : '8(f6.2,1X))
952     Format('<Residuals>: '8(f6.2,1X))
954     Format('<Cont. Residuals>: '8(f6.2,1X))
953     Format('Sigma Res. :'(1X,f6.2))
955     Format(75(1H*))

        END
c______________________________________________________________________
        Subroutine Remise(t,SC)

        integer*4 i,t,SC(7)

        Do i=1,t
           SC(i)=0
        EndDo

        return
        END
c____________________________________________________________________

      SUBROUTINE traitement(SC,ncont,nb,marg,Cont,DR,DC,sir,sqnb)

        integer*4 nb(7,51,1),i,j,SC(7),ncont(51,1),
     *sqnb(7,51,1)
      real*4 CA(7,8),WA(7,8),WG(8,1),CSG(8,51,1),a,A1,A2,DC(8),
     *CSA(7,8),DR(4),DL(8,1),sir(8),marg(8,51,1),Cont(8,51,1),
     *errcon(8,51,1)
        Integer*8 nComb
        common/com/ nSyn
        common/com3/WG,CSG,WA,CA,CSA
        
        common/com4/DL,errcon
        common/com5/nComb

        Do 95 mk=1,nSyn
        Do j=1,8
           A1=0.
           A2=0.
           Do i=1,7
              A1=A1+FLOATJ(SC(i))*CA(i,j)
           EndDo
           Do i=1,7
              A2=A2+FLOATJ(SC(i))*CA(i,j)*WA(i,j)/A1
           EndDo
           DR(j)=WG(j,mk)-A2
           a=abs(DR(j))

           If (a.gt.DL(j,mk)) Then
              Go To 95
           EndIf
           a=a+(DR(j))*(DR(j))
           sir(j)=a**0.5
        EndDo

       Do 90 l=1,51
       Do j=1,8
          A1=0.
          Do i=1,7
             A1=A1+FLOATJ(SC(i))*CSA(i,j)/100.
          EndDo
          DC(j)=(CSG(j,l,mk)-A1)
          a=abs(DC(j))
          If (a.gt.errcon(j,l,mk)) Go To 90
       EndDo

        ncont(l,mk)=ncont(l,mk)+1
        Do i=1,7
           nb(i,l,mk)=nb(i,l,mk)+SC(i)
           sqnb(i,l,mk)=sqnb(i,l,mk)+SC(i)*SC(i)
        EndDo
        Do j=1,8
           marg(j,l,mk)=marg(j,l,mk)+DR(j)
          Cont(j,l,mk)=Cont(j,l,mk)+DC(j)
       EndDo
 90     continue
 95     continue

100     continue
        return
        END
